/**
  *  \file client/cscreen.h
  *  \brief Control Screens
  *
  *  (c) 2001-2009 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_CSCREEN_H
#define PCC_V2_CLIENT_CSCREEN_H

#include "client/chart.h"
#include "cpluslib/smartptr.h"
#include "game/objtypes.h"
#include "resmgr/resmgr.h"
#include "ui/commands.h"
#include "ui/ui.h"
#include "cpluslib/holder.h"
#include "ui/skin.h"
#include "client/wtile.h"
#include "client/screens.h"

enum {
    cm_CS_ToggleSel = cm_ControlScreenFirst,
    cm_CS_Xchg                     // "X" - used internally
};

class WControlScreen;

class WObjectSelectionTile : public UIWidget {
    GObjectType&      type;
    SignalConnection  conn;
    int               current;

    void doUpdate();
 public:
    WObjectSelectionTile(GObjectType& type);
    ~WObjectSelectionTile();
    void  setCurrent(int id);

    /** Get the type we are selecting from. */
    GObjectType& getType() const
        { return type; }
    /** Get currently-selected object. */
    int getCurrent() const
        { return current; }

    void getLayoutInfo(LayoutInfo& info);

    int doStandardDialog(string_t title);
};

class WScannerChartWidget;

class WScanner : public UIWidget {
    WScannerChartWidget* chart;
    int sid, xpid, gpid;

    SignalConnection update_conn;
 public:
    WScanner();
    ~WScanner();

    void setChart(WScannerChartWidget& achart);
    void update();
    void drawContent(GfxCanvas& can);
    void getLayoutInfo(LayoutInfo& info);

    int getScannedPlanetId(bool exact) const { return exact ? xpid : gpid; }
    int getScannedShipId() const { return sid; }
    WScannerChartWidget* getChart() const { return chart; }
};

class WControlScreenSkin : public UISkin {
 public:
    const UIColorSet& getColors() const;
    void  outText(GfxCanvas& can,  int f, int x, int y, string_t text);
    void  outTextF(GfxCanvas& can, int f, int x, int y, int lim, string_t text);
    void  drawBackground(GfxCanvas& can, GfxRect r);
};

class WControlScreen : public UIFixedWidget {
    WControlScreenSkin skin;
    WScanner      scanner;
    WScreenId::ScreenId screen_id;    ///< Screen Id (function key number).

    SignalConnection pos_changed_conn;

    void setGeometry();
    void onPosChanged(int);
 protected:
    GObjectType&  type;

 private:
    /* tile positioning */
    int   tile_width;       // width of one column
    int   tile_y;           // current y
    int   tile_x;
    int   tile_new_y;       // y for new columns
    int   chart_y2;
    int   chart_x1;

    WScreenId return_value;

    WScannerChartWidget*      chart;
    UIGroup                   controls;
 protected:
    ObjectHolder holder;

    void addChart(WScannerChartWidget& chart);
    void addTile(WTile& tile);
    void addTile(WTile& tile, const char* title);
 public:
    WControlScreen(GObjectType& atype, WScreenId::ScreenId screen_id);
    ~WControlScreen();

    bool          handleEvent(const UIEvent& e, bool second_pass);
    bool          handleCommand(const UICommand& cmd, UIBaseWidget& sender);
    void          drawContent(GfxCanvas& can);
    void          setCurrent(int id);
    GObjectType&  getType() { return type; }
    GPoint        getScannerPosition() const;
    WScanner&     getScanner() { return scanner; }
    WScreenId::ScreenId getScreenId() const { return screen_id; }

    virtual void  update() = 0;

    WScreenId     runScreen();

    void          stopWithScreenId(WScreenId id);
};

class WControlScreenHeaderTile : public WObjectTypeObserverTile {
    int buttons;
    ObjectHolder holder;
 public:
    WControlScreenHeaderTile(GObjectType& type, int buttons);
    void drawData(GfxCanvas& can);

    virtual string_t getTitle() = 0;
    virtual string_t getSubtitle() = 0;
    virtual ResId getPictureId() = 0;

    enum {
        bName  = 1,              // N
        bSend  = 2,              // I
        bAuto  = 4,              // Auto
        bCycle = 8,              // ,->
        bAdd   = 16              // Add (Ins)
    };
};

class WHeadingTile : public WTile {
    string_t text;
 public:
    WHeadingTile(const string_t& s);
    WHeadingTile(const char* s);
    void drawData(GfxCanvas& can);
};

#endif
