/**
  *  \file client/dlg-draw.h
  *  \brief Drawing-related dialogs
  */
#ifndef PCC_V2_CLIENT_DLG_DRAW_H
#define PCC_V2_CLIENT_DLG_DRAW_H

#include "game/drawing.h"
#include "ui/ui.h"
#include "ui/window.h"

/** Color selector widget. Allows selection of a user color. Note that
    user colors are allowed to be zero (yielding invisible objects). 
    Hence we allow color 0 in our program interfaces, but we do not
    allow the user to select color 0. */
class WColorSelector : public UIWidget {
    /** User color index (not a palette index!). */
    int user_color;

    void moveBy(int delta);
    int  getFrameSize() const;
 public:
    WColorSelector();
    ~WColorSelector();

    void setColor(int user_color);
    int  getColor() const;

    void getLayoutInfo(LayoutInfo& info);
    void drawContent(GfxCanvas& can);
    bool handleEvent(const UIEvent& e, bool second_pass);

    enum {
        rCancel,
        rOK,
        rAdjacent
    };
    int doStandardDialog(string_t title, bool offer_adjacent);
};


class WMarkerTypeSelector : public UIWidget {
    /** Marker type. */
    int marker_type;

    void moveBy(int delta);
    int  getFrameSize() const;
 public:
    WMarkerTypeSelector();
    ~WMarkerTypeSelector();

    void setMarkerType(int marker_type);
    int  getMarkerType() const;

    void getLayoutInfo(LayoutInfo& info);
    void drawContent(GfxCanvas& can);
    bool handleEvent(const UIEvent& e, bool second_pass);

    bool doStandardDialog(string_t title);
};


/** New drawing selection dialog */
class WNewDrawingDialog : public UIWindow {
    // User settings:
    uint8 user_color;
    atom_t tag;
    GDrawingType type;

    // UI stuff:
    ObjectHolder h;
    void buildDialog();
 public:
    WNewDrawingDialog();
    ~WNewDrawingDialog();
    bool runDialog();

    void setColor(uint8 user_color);
    uint8 getColor() const;

    void setTag(atom_t tag);
    atom_t getTag() const;

    GDrawingType getType() const;
};

#endif
