/**
  *  \file client/screens.h
  *
  *  (c) 2001-2009 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_SCREENS_H
#define PCC_V2_CLIENT_SCREENS_H

#include "client/wtile.h"
#include "cpluslib/string.h"
#include "game/cargo.h"
#include "game/coord.h"
#include "game/objtypes.h"
#include "game/player.h"
#include "io/io.h"
#include "ui/widgets.h"

class GPlanet;

/* client/screens.cc */

/** Screen Identifier. Identifies a screen the user may be in, i.e. a major
    position in the main state machine. */
class WScreenId {
 public:
    enum ScreenId {
        NullScreen        = -1,
        RaceScreen        = 0,

        ShipScreen        = 1,
        PlanetScreen      = 2,
        BaseScreen        = 3,
        Starchart         = 4,

        ShipTaskScreen    = 11,
        PlanetTaskScreen  = 12,
        BaseTaskScreen    = 13
    };

    WScreenId();
    WScreenId(ScreenId screen_id, int id);
    WScreenId(GPoint pt);

    ScreenId getScreenId() const;
    int      getId() const;
    GPoint   getPos() const;
    bool     isNonNull() const;
    bool     isValid() const;

    string_t toString();

    static bool isValidScreenId(int n);

    bool operator==(const WScreenId& other) const;
    bool operator!=(const WScreenId& other) const;

 private:
    ScreenId screen_id;
    int x, y;
};

WScreenId doSelectObjectForScreen(WScreenId::ScreenId screen_id);


inline WScreenId::ScreenId
WScreenId::getScreenId() const
{
    return screen_id;
}

inline bool
WScreenId::isNonNull() const
{
    return screen_id != NullScreen;
}

inline bool
WScreenId::operator==(const WScreenId& other) const
{
    return screen_id == other.screen_id
        && x == other.x
        && y == other.y;
}

inline bool
WScreenId::operator!=(const WScreenId& other) const
{
    return !operator==(other);
}

/* client/scr-ship.cc */
int chooseShip();
WScreenId doShipScreen();

WTile* createShipScreenHeaderTile(GObjectType& type);

/* client/scr-planet.cc */
int choosePlanet();
WScreenId doPlanetScreen();

class WPlanetFCodeTile : public WObjectTypeObserverTile {
    ObjectHolder h;
 public:
    WPlanetFCodeTile(GObjectType& type);
    void drawData(GfxCanvas& can);
};

class WPlanetBaseTile : public WObjectTypeObserverTile {
    bool on_base;
    ObjectHolder h;
 public:
    WPlanetBaseTile(GObjectType& type, bool on_base);
    void drawData(GfxCanvas& can);
};

/* client/scr-base.cc */
int chooseBase();
WScreenId doBaseScreen();

/* client/scr-msg.cc */
int doInboxScreen();
int doSendOrReviseMessage();
int doSendMessage(string_t text, GPlayerSet receivers);

/* client/scr-env.cc */
class WPlanetMiningTile : public WTile {
    GPlanet&   planet;
    GCargoType ele;
    mp16_t     mines;
 public:
    WPlanetMiningTile(GPlanet& planet, GCargoType ele);
    void drawData(GfxCanvas& can);
    void setNumMines(mp16_t mines);
};

WScreenId doPlanetScan(int pid);

/* client/scr-mine.cc */
class WObjectTypeHeaderTile : public WObjectTypeObserverTile {
    UIButton button_next, button_prev;

    void onNext(), onPrev();
 public:
    WObjectTypeHeaderTile(GObjectType& type);
    void drawData(GfxCanvas& can);
    void getLayoutInfo(LayoutInfo& info);
};

WScreenId doMinefieldScreen();
int findNearestMinefield(GPoint pt);

/* client/scr-ufo.cc */
WScreenId doUfoScreen();
int findNearestUfo(GPoint pt);

#endif
