/**
  *  \file client/wcolorframe.cc
  *  \brief Color Frame Widget
  *  \author Stefan Reuther
  *
  *  (c) 2001-2009 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/wcolorframe.h"
#include "gfx/palette.h"
#include "ui/layout.h"
#include "ui/skin.h"

/** Construct blank color frame. This assigns a UIHBoxLayout::instance0, which will normally
    layout this widget to enclose its content snugly. If you really need it, you can assign
    a different layout. */
WColorFrame::WColorFrame()
    : UILayoutableWidget(UIHBoxLayout::instance0),
      color(cNone)
{ }

/** Set frame color. Triggers redraw if needed. */
void
WColorFrame::setColor(Color c)
{
    if (c != color) {
        color = c;
        drawWidget();
    }
}

void
WColorFrame::drawContent(GfxCanvas& can)
{
    UISkin& sk = getSkin();
    if (color == cNone) {
        sk.drawBackground(can, GfxRect(extent.x,            extent.y,            extent.w, 2));
        sk.drawBackground(can, GfxRect(extent.x,            extent.y+extent.h-2, extent.w, 2));
        sk.drawBackground(can, GfxRect(extent.x,            extent.y+2,          2,        extent.h-4));
        sk.drawBackground(can, GfxRect(extent.x+extent.w-2, extent.y+2,          2,        extent.h-4));
    } else {
        uint8 inner, outer;
        if (color == cYellow) {
            inner = COLOR_BRIGHTYELLOW;
            outer = COLOR_DARKYELLOW;
        } else if (color == cGreen) {
            inner = COLOR_GREENSCALE + 10;
            outer = COLOR_GREENSCALE + 8;
        } else {
            inner = COLOR_FIRE + 8;
            outer = COLOR_FIRE + 6;
        }
        GfxRect x = getExtent();
        sk.getParameters().color = standard_colors[outer];
        drawRectangle(can, sk, x);
        x.grow(-1, -1);
        sk.getParameters().color = standard_colors[inner];
        drawRectangle(can, sk, x);
    }
    can.update(getExtent());
}

void
WColorFrame::adjustSize(GfxRect& size, bool to_inner)
{
    if (to_inner)
        size.grow(-2, -2);
    else
        size.grow(+2, +2);
}
