/**
  *  \file game/objtypes.h
  *  \brief Object Type
  *
  *  (c) 2001-2009 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_OBJTYPES_H
#define PCC_V2_GAME_OBJTYPES_H

#include "cpluslib/string.h"
#include "cpluslib/sigslot.h"

class GUniverse;
class GObject;

class GObjectType {
    GObjectType(const GObjectType&);
    GObjectType& operator=(const GObjectType&);

    int current;
    GUniverse* universe;

 public:
    GObjectType();
    virtual ~GObjectType();

    // Base class interface:

    /** Initialize. Called by setUniverse(). This may perform any operations the
        object type has to do to adapt to the new universe. Users should call
        setUniverse() instead. Note that this might encounter a null universe. */
    virtual void init() = 0;
    /** Check whether an object exists. */
    virtual bool isExistant(int index) = 0;
    /** Get object by Id. */
    virtual GObject& getObject(int index) = 0;
    /** Get next object after index.
        \param index current object number; might be non-existant or 0
        \param mark iff true, only access marked objects
        \returns 0 if no object (only if no objects exist or mark requested and no object marked!) */
    virtual int getNext(int index, bool mark) = 0;
    /** Get previous object after index.
        \param index current object number; might be non-existant or 0
        \param mark iff true, only access marked objects
        \returns 0 if no object (only if no objects exist or mark requested and no object marked!) */
    virtual int getPrev(int index, bool mark) = 0;

    // Operations:

    void setCurrent(int index);
    int getCurrent() const;

    void setUniverse(GUniverse* univ);
    GUniverse* getUniverse() const;

    /** Position changed. Raised when a (possibly) different object is selected.
        Parameter is new index. */
    Signal1<int> sig_pos_changed;
};

class GIndexedObjectType : public GObjectType {
    int count;
    int max;

 public:
    GIndexedObjectType(int max);

    // GObjectType:
    void init();
    int getNext(int index, bool mark);
    int getPrev(int index, bool mark);

    // GIndexedObjectType:
    int getCount() const;
};

#endif
