/**
  *  \file gfx/update.h
  *  \brief Update Management
  *
  *  (c) 2001-2009 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GFX_UPDATE_H
#define PCC_V2_GFX_UPDATE_H

#include "gfx/gfx.h"

/*! \class GfxUpdateOptimizer
    \brief Update Optimizer

    This canvas collects GfxCanvas::update(), and performs them as one
    big update() call when told to (or upon destruction). All other
    function calls are handed through to the parent canvas as given to
    the constructor. */

class GfxUpdateOptimizer : public GfxCanvas {
    GfxCanvas& parent;
    GfxRect update_region;

 public:
    GfxUpdateOptimizer(GfxCanvas& parent);
    ~GfxUpdateOptimizer();

    void doUpdates();
    void clearUpdateRegion(GfxRect& region);

    void drawHLine(int x1, int y1, int x2, uint32 color, uint8 pat, alpha_type alpha);
    void drawVLine(int x1, int y1, int y2, uint32 color, uint8 pat, alpha_type alpha);
    void drawPixel(int x, int y, uint32 color, alpha_type alpha);
    void drawBar(GfxRect rect, uint32 color, uint32 bg, const GfxFillPattern& pat, alpha_type alpha);

    void blitSurface(int x, int y, GfxCanvas& src, GfxRect rect);
    void blitPattern(GfxRect rect, int x, int y, int bytes_per_line, const uint8* data, uint32 color, uint32 bg, alpha_type alpha);

    bool lock();
    void unlock();
    void update(GfxRect r);

    GfxRect computeClipRect(GfxRect r);
    GfxPoint getSize();
    int getBitsPerPixel();

    uint32 getPixel(int x, int y);
};

#endif
