#include "io/io.h"
#include "u/t_io.h"

#include <cstdlib>

struct test_struct {
    int16 small;
    char  string[10];
    int32 large[2];
};

const TStructureParser parser[] = {
    { T_INT16,   1, offsetof(test_struct, small), 0, 0 },
    { T_STRING, 10, offsetof(test_struct, string), 0, 0 },
    { T_INT32,   2, offsetof(test_struct, large), 0, 0 },
    { T_END,     0, 0, 0, 0 }
};

const char encoded[] = {
    0x10, 0x20,
    'h', 'e', 'l', 'l', 'o', ' ', ' ', ' ', ' ', ' ',
    0xfe, 0xff, 0xff, 0xff, 0x10, 0x20, 0x30, 0x40
};

void
IoIoTestSuite::testIO()
{
    test_struct d;
    char new_encoded[sizeof encoded];

    TS_ASSERT_EQUALS(sizeof encoded, 20);
    TS_ASSERT_EQUALS(getStructureSize(parser), 20);
    TS_ASSERT_EQUALS(getStructure(encoded, &d, parser), encoded + 20);
    TS_ASSERT_EQUALS(d.small, 0x2010);
    TS_ASSERT_EQUALS(d.string[0], 'h');
    TS_ASSERT_EQUALS(getBasicString(d.string), "hello");
    TS_ASSERT_EQUALS(d.large[0], -2);
    TS_ASSERT_EQUALS(d.large[1], 0x40302010);
    TS_ASSERT_EQUALS(storeStructure(new_encoded, &d, parser), new_encoded + 20);
    TS_ASSERT_EQUALS(std::memcmp(encoded, new_encoded, 20), 0);
    TS_ASSERT_EQUALS(getUint16(encoded+12), 0xFFFEU);
    TS_ASSERT_EQUALS(getInt16(encoded+12), -2);
    TS_ASSERT_EQUALS(getUint32(encoded+12), 0xFFFFFFFEU);
    TS_ASSERT_EQUALS(getInt32(encoded+12), -2);

    // FIXME: missing: getPartialStructure
}
